# AutoNap shell module $Revision: 1.15 $
# last change made $Date: 2001/03/23 22:48:46 $

import MP3::Napster;

package AutoLib::Shell;

BEGIN {
	use Exporter   ();
	use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

	$VERSION = 1.0;

	@ISA          = qw(Exporter);
	@EXPORT       = qw(
			   &make_global
			   &cmd_check &cmd_help &cmd_search &cmd_get 
	                   &cmd_downs &cmd_ddown &cmd_quit &cmd_stream
			   &cmd_browse &cmd_last &cmd_blast &cmd_ping
			   &display_results 
			  );
	%EXPORT_TAGS  = ( );

	}

use AutoLib::Callbacks;
use AutoLib::Cache;
use AutoLib::Trap;
use MP3::Napster::User;
use IO::File;
use Term::ReadKey;
use Term::ANSIColor qw (:constants);
use warnings;
use strict;
use vars     @EXPORT_OK;


# some globs
our ($NAP, $DIR, $PLAYER, $SCR_WIDTH, $SCR_HEIGHT);



#-------------------------------
# make global variables 
# accessible throughout Shell.pm
#-------------------------------
sub make_global {
	($NAP, $DIR, $PLAYER, $SCR_WIDTH, $SCR_HEIGHT) = @_;
}


#-----------------------
# provide help commandos
#-----------------------
sub cmd_help {	
	my $arg = shift;
	my @ret;

	# remove leading and trailing whitespaces
	$arg =~ s/^\s+|\s+$//;
		
	if ($arg eq 'help' or not $arg) {
		@ret  = (
			GREEN.
			"%-8.8s%-8.8s%-8.8s%-8.8s\n".
			"%-8.8s%-8.8s%-8.8s%-8.8s\n".
			"%-8.8s%-8.8s%-8.8s%-8.8s",	
			"check", "blast","browse", "ddown", 
			"downs", "get", "help", "last", 
			"ping", "quit", "search", "stream"
			);
	}
	elsif ($arg eq 'blast') {
		@ret = GREEN."blast <num> - just like 'last' but accessing browse history";
	}
		
	elsif ($arg eq 'browse') {
		@ret = GREEN."browse <user> - show list of <user>'s shared songs";
	}
	elsif ($arg eq 'check') {
		@ret = GREEN."check <song_num> - show info on given song more detailed";
	}
	elsif ($arg eq 'ddown') { 
		@ret = (
		       GREEN."%-6.6s%-11.11s - %-s\n%-6.6s%-11.11s - %-s\n%-6.6s%-11.11s - %-s",
		       "ddown", "<song_num>", 
		       "deletes download of given song",
		       " ",     "<song_list>",
		       "deletes download of songs 'n-m' ('all' for all)",
		       " ",     "<song_enum>",
		       "deletes download of songs 'n,m,o...'"
		       );
	}
	elsif ($arg eq 'downs') {
		@ret = GREEN."downs - info on currently running downloads";
	}
	elsif ($arg eq 'get') {
		@ret = (
			GREEN."%-4.4s%-11.11s - %-s\n%-4.4s%-11.11s - %-s\n%-4.4s%-11.11s - %-s",
			"get", "<song_num>",
			"tries to download given song",
			" ",   "<song_list>",
			"tries to download song 'n-m' ('all' for all)",
			" ",   "<song_enum>",
			"tries to download songs 'n,m,o...'"
			);
	}
	elsif ($arg eq 'search') {
		@ret = (
			GREEN."%-7.7s%-28.28s - %-s\n%-7.7s%-31.31s%-s",
			"search", "<pattern>[||<min_linespeed>]",
			"searches for files containing <pattern>",
			" ", " ",
			"unfortunately nap-servers don't yet support regexps"
			);
	}
	elsif ($arg eq 'stream') {
		@ret = GREEN."stream <song_num> - get and listen to song at one time";
	}
	elsif ($arg eq 'last') {
		@ret = GREEN."last [<num>] - display last search's result\n".
		             "               if <num> is given, the <num>th search's result\n".
			     "               is dislayed. 'last 0' also returns last search";
	}
	elsif ($arg eq 'ping') {
		@ret = GREEN."ping <user> [<wait>] - ping user and wait <wait> secs for reply\n".
		             "                       if <wait> is not given, defaults to 3";
	}
	elsif ($arg eq 'quit') {
		@ret = (
			GREEN."%-6.6s - %-s",
			"quit", "haha, have a guess"
			);
	}
	else {
		my $l = length($arg);
		$l++;
		@ret = (
			GREEN."%-".$l.".".$l."s - %-s\n%-".$l.".".$l."s   %-s",
		        $arg, "unknown command",
			" ", "try 'help' for info on available commands"
			);
	}
	
	return @ret;
}

#-------------------------
# provides search function
#-------------------------
sub cmd_search { 
	my $p = shift;

	# remove leading and trailing whitespaces
	# just in case...
        $p =~ s/^\s+|\s+$//;
	
	my ($artist, $linespeed) = split /\|\|/, $p;
	if (defined($linespeed)) {
		$linespeed =~ s/(.*)/LINK_$1/; 
	}
	else {
		$linespeed = "LINK_DSL";
	}
	my @found = $NAP->search(artist=>$artist,
		            linespeed=>$linespeed);	

	# unless search retrieved no results, cache
	cache_search(@found) unless @found == 0;
	display_results(@found);
	return @found;
}

#--------------------------------------
# browse another user's song-repository
#--------------------------------------
sub cmd_browse {
	my $user = shift;
	
	# remove leading whitespaces
	$user =~ s/^\s+//;
		
	if (not $user) {
		printf BLUE."-----\n";
		printf RED."%-s", 
		       "You should give a user's name to browse his archive"; 
		printf BLUE."\n-----".RESET."\n";
		return;
	}
	
	my @found = $NAP->browse($user);
	
	if ($NAP->error) {
		printf RED.$user.": ".$NAP->error." (Warning).".RESET."\n";
	}
	
	# unless browse retrieved no results, cache
	unless (@found == 0) {
		cache_browse(@found) unless @found == 0;
		display_results(@found);
		return @found;
	}
	else  {
		printf BLUE."-----\n";
		printf RED."Either the user shares no songs or concetion to him timed out.";
		printf BLUE."\n-----".RESET."\n";
		return 0;
	}
}

#-------------
# pings a user
#-------------
sub cmd_ping {
	my $arg = shift;

	# remove leading whitespaces
        $arg =~ s/^\s+//;
	
	if (not $arg) {
		printf BLUE."-----\n";
	        printf RED."%-s",
                           "You should give a user's name to send the ping to";
	        printf BLUE."\n-----".RESET."\n";
	        return 0;
	}
	
	my ($user, $wait) = split /\s+/, $arg;
	$wait ||= 3;

	if ($NAP->ping($user, $wait)) {
                printf BLUE."-----\n".
		       GREEN."Received response within $wait secs. Looks good.".
		       BLUE."\n-----".RESET."\n";
	}
	else {
		printf BLUE."-----\n".
		       RED."No PONG reveived within $wait secs.".
		       BLUE."\n-----".RESET."\n";
	}							       
}

#------------------------------
# displays last search's result
#------------------------------
sub cmd_last {
	my $arg = shift;

	# remove leading and trailing whitespaces
        $arg =~ s/^\s+|\s+$//;
		
	if ($arg =~ /^\d+$/ and $arg <= num_of_search()) {
		$arg--;
		my @f = get_from_cache("search", $arg);
		display_results(@f);
		return @f;
	}
	elsif ($arg eq 'last') {
		my @f = get_from_cache("search", -1);
		display_results(@f);
		return @f;
	}
	elsif ($arg =~ /^\d+$/ and $arg > num_of_search()) {
		printf BLUE."-----\n";
		printf RED."There only have been ".num_of_search()." searches so far.";
		printf BLUE."\n-----".RESET."\n";
		return undef;
	}
	else {
		printf BLUE."-----\n";
		printf RED."Argument of 'last' either must be numeric or none at all.";
		printf BLUE."\n-----".RESET."\n";
		return undef;
	}
}

#-----------------------------
# Display last browse's result
#-----------------------------
sub cmd_blast {
	my $arg = shift;

	# remove leading whitespaces
        $arg =~ s/^\s+|\s+$//;
		
	if ($arg =~ /^\d+$/ and $arg <= num_of_browse()) {
		$arg--;
		my @f = get_from_cache("browse", $arg);
		display_results(@f);
		return @f;
	}
	elsif ($arg eq 'blast') {
		my @f = get_from_cache("browse", -1);
		display_results(@f);
		return @f;
	}
	elsif ($arg =~ /^\d+$/ and $arg > num_of_browse()) {
		printf BLUE."-----\n";
		printf RED."There only have been ".num_of_browse()." browses so far.";
		printf BLUE."\n-----".RESET."\n";
		return undef;
	}
	else {
		printf BLUE."-----\n";
		printf RED."Argument of 'blast' either must be numeric or none at all.";
		printf BLUE."\n-----".RESET."\n";
		return undef;
	}
}



#-----------------------------
# details about specified song
#-----------------------------
sub cmd_check {
	my ($arg, @found) = @_;
	my $song = undef;

	# remove leading and trailing whitespaces 
        $arg =~ s/^\s+|\s+$//;
	
	unless ($arg > @found or $arg < 1) {
		$song = $found[$arg - 1];
	}
	else {
		printf BLUE."-----\n";
		printf RED."No such item ($arg) to check";
		printf BLUE."\n-----".RESET."\n";
		return 0;
	}
	my ($title, $size, $bitrate, $freq, $owner, $length, $hash, $address, $link) =
		(
			$song->name,
			$song->size,
			$song->bitrate,
			$song->freq,
			$song->owner,
			$song->length,
			$song->hash,
			$song->address,
			$song->link
		);
		
	$size = $size / 1024 / 1024;
	$size = sprintf "%-4.2f", $size;
	
	my $sec = $length % 60;
	my $min = ($length - $sec) / 60;
	$length = $min.":".$sec;

	printf BLUE."-----\n";
	printf BLUE."Song:   ".GREEN.$title."\n";
	printf BLUE."Owner:  ".GREEN.$owner."\n";
	printf BLUE."Link:   ".GREEN.$link."\n";
	printf BLUE."Length: ".GREEN.$length." min\n";
	printf BLUE."Size:   ".GREEN.$size." mb\n";
	printf BLUE."Freq:   ".GREEN.$freq." Hz\n";
	printf BLUE."Bitr:   ".GREEN.$bitrate." kb/s\n";
	printf BLUE."MD5:    ".GREEN.$hash."\n";
	printf BLUE."Addr:   ".GREEN.$address;
	printf BLUE."\n-----".RESET."\n";
	return 1;
}

#------------------------------
# provides downloading function
#------------------------------
sub cmd_get {	
	my @songs;
	my ($arg, @found) = @_;
	
	# remove leading and trailing whitespaces 
	$arg =~ s/^\s+//;
	
	if ($arg > @found or $arg < 0) {
		printf BLUE."-----\n".
		       RED."No such item ($arg) to get".
		       BLUE."\n-----".RESET."\n";
	}
	
	elsif ($arg =~ /^(\d{1,3},)+\d{1,3}$/) {
		@songs = split /,/, $arg; 
	}
	elsif ($arg =~ /^(\d)+-(\d)+$/) {
		my ($first, $last) = split /-/, $arg;
		for (my $i = 0; $i <= $last - $first; $i++) {
			$songs[$i] = $first + $i;
		}
	}
	elsif ($arg =~ /^\d+/) {
		if ($arg > 0) {
			push @songs, $arg;
		}
		else {
			printf BLUE."-----\n".
			       RED."Invalid song number ($arg)".
			       BLUE."\n-----".RESET."\n";
		}
	}
	else
	{
		printf RED."%-s", "$arg: Invalid arguments for 'get'".RESET."\n";
	}
	
	for my $s (@songs) {
		my $curSong = $found[$s - 1]->{'name'};
		my $handle = $DIR."/".$curSong;
		my $download;
		if ($found[$s - 1]->owner->ping) {
			$download = $NAP->download($found[$s - 1], $handle);
			$download->interval(2000) if (defined($download));
		}
		else {
			printf BOLD.RED."%-s\n", "No PONG received for $curSong.".RESET;
		}
	}			
	
}

#------------------------------
# delete a running download job
#------------------------------
sub cmd_ddown {
	my $no = shift;

	# remove leading whitespaces from $arg
	$no =~ s/^\s+//;
	
	my @transfers = $NAP->downloads;
	if ($no < 1 or $no > @transfers) {
		printf BLUE."-----\n".
		       RED."No such item ($no)".
		       BLUE."\n-----".RESET."\n";
	}
	else {
		$transfers[$no - 1]->abort;
	}
}

#---------------------------------------
# returns infos on running download jobs
#---------------------------------------
sub cmd_downs {
	my $n = shift;

	# remove leading whitespaces from $arg
	$n =~ s/^\s+//;
	
	my $ord = 1;
	printf BLUE."-----\n";
	for my $song ($n->downloads) {
		my ($text, $transferred, $size, $perc);
		$text        = $song->asString;
		$transferred = $song->transferred;
		$size        = $song->size;
		no AutoLib::Trap; #{
			eval { $perc = $transferred / $size * 100; };
		#}
		
		use AutoLib::Trap;

		if ($@) {
			printf GREEN."%-4s - %-s\n%-s\n",
			       "($ord)", $song->asString,
			       "stats unkown";
		}
		else {
			printf  GREEN."%-4s - %-s\n%-d / %-d = %-2.2f%%\n", 
		        	"($ord)", $song->asString,
			 	$transferred, $size, $perc;
		}
		$ord++;
	}
	printf BLUE."-----".RESET."\n";
}

#---------------------------------------------------------------------
# tees the song into a file and an mp3player specified in a configfile
#---------------------------------------------------------------------
sub cmd_stream {
	my ($arg, @found) = @_;
	my $song;
	my $curSong;

	# remove leading whitespaces
	$arg =~ s/^\s+//;
		
	if ($arg =~ /^\d+/ and $arg > 0) {

		$song = $found[$arg - 1];
		$curSong = $song->name;
		
		my $handle = IO::File->new("| tee $DIR/".quotemeta($curSong)." | $PLAYER");
			                         
		my $download;
		if ($song->owner->ping) {
			$download = $NAP->download($song, $handle);
			$download->interval(2000) if (defined($download));
		}
		else {
			printf BLUE."-----\n";
			printf BOLD.RED."%-s\n", "No PONG received for $curSong.";
			printf BLUE."\n-----".RESET."\n";
		}
	}
	else {
		printf BLUE."-----\n".
		       RED."Invalid song number".
		       BLUE."\n-----".RESET."\n";
	}	
}

#----------------------------------------
# quit connection and destroy objects
# MP3::Napster kindly handles that for us
# make sure to reset terminal
#----------------------------------------
sub cmd_quit {
	ReadMode 0;
	printf UNDERLINE.GREEN."Thanks for using AutoNap!";
	printf RESET."\n";
	exit(0);
}

#---------------------------------
# Print results formatted properly
#---------------------------------
sub display_results {
	
	my $key;
 
	printf GREEN."%-4s".BLUE."|".GREEN."%-50.50s".BLUE."|".GREEN."%-8s".
	       BLUE."|".GREEN."%-3s".BLUE."|".GREEN."%-6s".BLUE."|".GREEN."%-8s\n", 
	       "No.", "Song" , "Size", "bit", "len", "speed";
	print BLUE;
	printf "-" x 4 ; printf "|"; 
	printf "-" x 50; printf "|"; 
	printf "-" x 8 ; printf "|";
	printf "-" x 3 ; printf "|";
	printf "-" x 6 ; printf "|";
	printf "-" x 8 ; printf RESET."\n";
	
	ReadMode 4;	

	my @found = @_;
	my $c = 1;
	for (@found) {

		my (
		$name, 
		$size,
		$bitrate,
		$seconds, $minutes, $length, 
		$speed
		);
	
		$name = substr $_->{'name'}, 0, -4;
		$seconds = $_->{'length'} % 60;
		$minutes = ($_->{'length'} - $seconds) / 60;
		$length = $minutes;
		if ($seconds > 9) {
			$length = $length.":".$seconds;
		}
		else {
			$length = $length.":0".$seconds;
		}
		$size = $_->{'size'} / 1024 / 1024;
		$bitrate = $_->{'bitrate'};
		$speed = LineSpeed($_->{'link'});
	
		printf "%-4d".BLUE."|".RESET."%-50.50s".BLUE."|".RESET."%-4.1f%3.2s ".
		       BLUE."|".RESET."%-3d".BLUE."|".RESET."%-6.6s".BLUE."|".RESET."%-8s\n",
			$c,
			$name,
			$size,"mb",
			$bitrate,
			$length,
			$speed;
		$c++;
	 
		if ($c % ($SCR_HEIGHT - 2) == 0) {
			
			LABEL_G:
			
			print GREEN, "-- any key to continue (q to quit listing) --", RESET;
			while (not defined($key = ReadKey(-1))) {
	    			# wait
			}
			print "\n";
			if ($key eq 'g' or $key eq 'x') {
				ReadMode 0;
				printf BLUE."Give song number: ";
				my $num = <STDIN>;
				printf RESET."\n";
				last if $num eq "\n";
				cmd_get($num, @found);
				last if $key eq 'x';
				ReadMode 4;
				goto LABEL_G;
			}
			if ($key eq 'q') {
				last;
			}
    		}
	}
	ReadMode 0;
	return 1;
}

#----------------------------------
# conversion form line code to text
#----------------------------------
sub LineSpeed {

        my $l = shift;
        my %speedtable = (
                0  => '???',
                1  => '14K',
                2  => '28K',
                3  => '33K',
                4  => '56K',
                5  => '64K',
                6  => '128K',
                7  => 'Cable',
                8  => 'DSL',
                9  => 'T1',
                10 => 'T3',
	                );
        return $speedtable{$l};

}

1;
