#!perl

# AutoNap $Revision: 1.15 $ 
# last change made $Date: 2001/03/23 22:47:14 $

use strict;
use Term::ReadKey;
use Term::ANSIColor qw (:constants);
use MP3::Napster;
use AutoLib::Net;
use AutoLib::Shell;
use AutoLib::Callbacks;
use AutoLib::Conffile qw(
			$ACC_NETWORK         $ACC_USER               $ACC_PASS               
			$ACC_DOWNLOAD_DIR
			$MISC_PLAYER
       	 		$DOWN_MIN_LINESPEED  $DOWN_MIN_BITRATE       $DOWN_MAX_SIMULTANEOUS
       	 		$DOWN_TIMEOUT        $DOWN_MIN_FILE_LENGTH
       			$SEARCH_PATTERN
       			$RND                 @RND_EXCLUDE_PATTERN	
			&parse_configfile
			);
use AutoLib::Cache     qw(
			@CACHE_SEARCH @CACHE_BROWSE
			&cache_search &cache_browse
			);
use AutoLib::Trap;

print RESET;

my 	(
	%OPT, %TRASH,
	$term, $prompt, $OUT,
	$key, $value,
	$nap,
	@found);

#----------------------------
# Parse commandline arguments 
# I don't know what's wrong
# with Getopt::Std and 
# Getopt::Long, but these two
# do not suffice
#----------------------------

my $cmd_length = @ARGV;
for (my $i = 0; $i < $cmd_length; $i++) {

	# it is an option requiring argument
	if ($ARGV[$i] =~ /^-[bdfFlnpPsSu]$/) {
		$OPT{$ARGV[$i]} = $ARGV[$i+1] unless $ARGV[$i+1] =~ /^-/;
	}
	
	# it is the -x option
	elsif ($ARGV[$i] =~ /^-x$/) {  
		# go to next element
		my $pos = $i + 1;  
		my $x_arg;
		# loop until it is again an option
		while (defined($ARGV[$pos]) and  $ARGV[$pos] !~ /^-.$/) {  
			$x_arg .= $ARGV[$pos]." ";
			$pos++;
		}
		$OPT{$ARGV[$i]} = $x_arg;
	}
	elsif ($ARGV[$i] =~ /^-[hmRr]/) {
		$OPT{$ARGV[$i]} = 1;
	}
}

my $h           = $OPT{'-h'};
my $file        = $OPT{'-f'};
my $server      = $OPT{'-S'};
my $meta        = $OPT{'-m'};
my $network     = $OPT{'-n'};
my $user        = $OPT{'-u'} ||= "prompt_me";
my $pass        = $OPT{'-p'} ||= "prompt_me";
my $register    = $OPT{'-R'};
my $down_dir    = $OPT{'-d'} ||= "/tmp";
my $player      = $OPT{'-P'} ||= "mpg123 -";
my $linespeed   = $OPT{'-l'};
my $bitrate     = $OPT{'-b'};
my $no_of_downs = '';
my $file_length = $OPT{'-F'};
my $song        = $OPT{'-s'};
my $random      = $OPT{'-r'};
my @rnd_pattern = ();
my $not_these   = $OPT{'-x'} ||= 'britney spears::backstreet boys';

# besides help, this will gracefully ignore any illegal input

if ($h or (keys %OPT) == 0) {
	help(); 
	# and quit
}

my @peers       = get_servers("www.napigator.com", 80);

#----------------------
# make some conversions
#----------------------

$player .= " -" unless $player =~ /\s-$/;
my @server = split $server, "::" if $server;
my @not_these = split $not_these, "::" if $not_these;

# fill @rnd_pattern with alphanums
for my $n (48..57) {
	push @rnd_pattern, chr($n);
}
for my $n (97..122) {
	push @rnd_pattern, chr($n);
}

# remove suffixed slash from path

#----------------------------------
# Read options given in configfile.
# However, these wont override 
# commandline-switches.
#----------------------------------

if ($file) {
	parse_configfile($file);

	$network       = $ACC_NETWORK      			unless $network;
	$user          = $ACC_USER        	 		unless $user;
	$pass          = $ACC_PASS         			unless $pass;
	$down_dir      = $ACC_DOWNLOAD_DIR 			unless $down_dir;
	$player        = $MISC_PLAYER      			unless $player eq 'mpg123 -';
	$linespeed     = $DOWN_MIN_LINESPEED                    unless $linespeed;
	$linespeed =~ s/.*/LINK_$&/;
	$bitrate       = $DOWN_MIN_BITRATE			unless $bitrate;
	$no_of_downs   = $DOWN_MAX_SIMULTANEOUS;
	$file_length   = len_to_secs($DOWN_MIN_FILE_LENGTH)     unless $file_length;
	$song          = $SEARCH_PATTERN                        unless $song;
	$random        = $RND;
	@not_these     = @RND_EXCLUDE_PATTERN                   unless @not_these;
}

# for actual login, we only need ip and port

for my $p (@peers) {
	my ($p_ip, $p_port) = ($p->{'ip'}, $p->{'port'});
	my $connect_string = $p_ip.":".$p_port;
	push @server, $connect_string;
}

# you want to register?

if ($register) {
	$network ||="whatever server we get first";
	my $m = "Trying Registration on $network: ";
	
	#else {
		# my $m = "Trying Registration on whatever server we will get first: ";
	#}
	printf UNDERLINE.GREEN;
	printf "-" x (length($m) - 1);
	printf "\n".$m."\n";
	printf "-" x (length($m) - 1);
	printf RESET."\n";
	if (register_account($network, @peers)) {
	      exit(0);
	}
	# damn!
	else {
		printf RED."Error: Couldn't register".RESET."\n";
		exit(1);
	}
}
					
	

# if users wants to be prompted,
# well, prompt him.

$user = get_user() if $user eq 'prompt_me';
$pass = get_pass() if $pass eq 'prompt_me';

	    
#---------------------
# all went fine so far 
#---------------------
shift @server if (not $meta);
for my $p (@peers) {
	my ($p_ip,
	    $p_port,
	    $p_network,
	    $p_users,
	    $p_songs,
	    $p_gigs,
	    $p_host,
	    ) = 
	    ($p->{'ip'},
	     $p->{'port'},
	     $p->{'network'},
	     $p->{'users'},
	     $p->{'songs'},
	     $p->{'gigs'},
	     $p->{'host'}
	    );
	    
	if (defined ($network)) {
		next unless $p_network eq $network;
	}
	
	printf "Trying ".GREEN.$p_host.RESET." on port ".
	        BLUE.$p_port.RESET." ($p_ip:$p_port)...\n";
	if (not $meta) {
		$nap = MP3::Napster->new(-server => $p_ip.":".$p_port); 
		if (!defined ($nap)){
			printf STDERR "Error: Server unreachable\n";
			next;
		}
	} 
	else {
		$nap = MP3::Napster->new(-meta => $p);
	}
	if (not $nap->login($user, $pass) or $@) {
    	 	printf BLUE."Couldn't connect to $p_ip:$p_port: ".RED.$nap->error.
		       RESET."\n";
	}
	else {
  		$nap->download_dir($down_dir);
		printf UNDERLINE.GREEN."Success: Logged in with nickname ".
		       UNDERLINE.BLUE.$nap->nickname."!".RESET."\n";
		last;
	}
}

if (not defined($nap)) {
    	printf STDERR "Error: Couldn't create \$nap-object. That's out of your hands. (AutoNap)\n";
	exit(1);
}


if (exists $OPT{'-s'}) {
	@found = $nap->search(artist=>$song, 
			      bitrate=>"at least $bitrate", 
			      linespeed=>"at least $linespeed");
	display_results(@found);
}

#----------------------------------------
# pass those variables unlikely to change
# to Shell.pm
# we need more of that later if we ever
# want to keep on maintining the code
#----------------------------------------

# make AutoNap fit the screen vertically (nifty)
my ($screen_width, $screen_height, undef, undef) = GetTerminalSize();
printf UNDERLINE.GREEN."Applying screensize of $screen_width x $screen_height...".
       RESET."\n";

make_global($nap, $down_dir, $player, $screen_width, $screen_height);

#------------------------------------
# add callbacks and command_processor
# to napster object                  
#------------------------------------

$nap->callback(MOTD, \&CLB_message_of_day);
$nap->callback(TRANSFER_STARTED, \&CLB_transfer_start);
$nap->callback(TRANSFER_DONE, \&CLB_transfer_end);
$nap->callback(TRANSFER_STATUS, \&CLB_transfer_status);
$nap->command_processor(\&auto_nap_shell, \*STDIN);

$nap->run;

#----------
# sume subs 
#----------

# the actual shell
sub auto_nap_shell {
	shift;
	my $input = shift;
	$input =~ s/\s+$//;
		# cmd_help
		if ($input =~ /^help/){
			$input  =~ s/(help)(.*)/$2/;
			my @output = cmd_help($input);
			printf BLUE."-----\n";
			printf @output;
			printf BLUE."\n-----".RESET."\n";
		}
		
		# cmd_quit
		elsif ($input =~ /^quit$/) {
		   	cmd_quit();
		}

		# cmd_get
		elsif ($input =~ /^get$|^get /) {
			$input =~ s/(get)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg();
			}
			else {
				cmd_get($input, @found);
			}
		}

		# cmd_search
		elsif ($input =~ /^search$|^search /) {
			$input =~ s/(search)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
                                missing_arg();
                        }
			else {	
				@found = cmd_search($input);
			}
		}

		# cmd_browse
		elsif ($input =~ /^browse$|^browse /) {
			$input =~ s/(browse)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
                        	missing_arg();
                        }
			else {		
				if(defined(my $b = cmd_browse($input))) {
					@found = $b;
				}
			}
		}

		# cmd_ping
		elsif ($input =~ /^ping$|^ping /) {
			$input =~ s/(ping)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
		                missing_arg();
		        }
			else {							
				cmd_ping($input);
			}
		}
		
		# cmd_ddown
		elsif ($input =~ /^ddown$|^ddown /) {
			$input =~ s/(ddown)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
                                missing_arg();
                        }
			else {									
				cmd_ddown($input);
			}
		}

		# cmd_downs
		elsif ($input =~ /^downs/) {
			cmd_downs($nap);
		}
		
		# cmd_check
		elsif ($input =~ /^check$|^check /) {
			$input =~ s/(check )(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
                                missing_arg();
                        }
			else {									
				cmd_check($input, @found);
			}
		}

		# cmd_last
		elsif ($input =~ /^last/) {
			$input =~ s/(last )(.*)/$2/;
			my @tmp = cmd_last($input); 
			@found = @tmp unless not defined $tmp[0];
		}

		# cmd_blast
		elsif ($input =~ /^blast/) {
			$input =~ s/(blast )(.*)/$2/;
			my @tmp = cmd_blast($input);
			@found = @tmp unless not defined $tmp[0];
		}

		# cmd_stream
		elsif ($input =~ /^stream$|^stream /) {
			$input =~ s/(stream)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
                                missing_arg();
                        }
			else {									
				cmd_stream($input, @found);
			}
		}
		
		# shell commands
		elsif ($input =~ /^!/) {
			$input =~ s/(!)(.*)/$2/;
			eval { system($2); };
		}

		# nothing
		elsif ($input =~ /^\s*$/) {
			# do nothing
		}

		# cmd_unkown
		else {
			printf BLUE."-----\n";
			printf RED."$input - Unkown command. Type 'help' in case you feel lost."; 
			printf BLUE."\n-----".RESET."\n";
		}
		return;
}

#--------------------------------
# converts song length to seconds
#--------------------------------
sub len_to_secs {
	my $t = shift;
	my ($min, $sec) = split /:/, $t;
	return $min * 60 + $sec;
}

#---------------------
# standard error reply
#---------------------
sub missing_arg {
	printf BLUE."-----\n".
	       RED."Argument(s) required".
	       BLUE."\n-----".
	       RESET."\n";
}

#-------------	
# get username
#-------------
sub get_user {
        printf "give username: ";
	chomp ($user = <STDIN>);
	return $user;
}

# get password
sub get_pass {
	printf "give password: ";
	# make concole non-echoing
	system "stty -echo </dev/tty" unless $ENV{EMACS};
	chomp ($pass = <STDIN>);
	system "stty echo </dev/tty" unless $ENV{EMACS};
	printf "\n\n";
	return $pass;
}

#------------------------------------------	
# try to register a new account on $network
#------------------------------------------
sub register_account {
	my ($network, @peers) = @_;
	
	my ($user, $pass) = (get_user(), get_pass());
	my %details = ( 
		link  => 'LINK_DSL',
		email => 'schwachmat@yahoo.com'
		);
		
	my $email;
		
	for my $p (@peers) {
		my ($p_ip,
	    	$p_port,
		    $p_network,
		    $p_users,
		    $p_songs,
		    $p_gigs,
		    $p_host,
		    ) = 
		    ($p->{'ip'},
		     $p->{'port'},
		     $p->{'network'},
		     $p->{'users'},
		     $p->{'songs'},
		     $p->{'gigs'},
		     $p->{'host'}
		    );
		    
		if (defined ($network)) {
			next unless $p_network eq $network;
		}
		else {
			printf RED."Error: You must specify a network for registering.".
			       RESET."\n";
			exit(1);
		}

		printf "Trying registration at ".GREEN.$p_host.RESET." on port ".
		        BLUE.$p_port.RESET." ($p_ip:$p_port)...\n";
		if (not $meta) {
			$nap = MP3::Napster->new(-server => $p_ip.":".$p_port); 
			if (!defined ($nap)){
				printf STDERR "Error: Server unreachable\n";
				next;
			}
		}
		if (defined($nap)) {
			$email = $nap->register($user, $pass, \%details);
			if (not $nap->error) {
				printf BLUE."Registration successfull since there's no ".
				            "error message from the server.";
				if (not $email) {
			       	       printf BLUE."\nBut no email address either, ".
				       	      " so hope for the best!".
					      RESET."\n";
				}
				else {
				       BLUE."$email your email address assigned to you ".
				            "by the server.".RESET."\n";
				}
				return 1;
			}
			else {
				printf RED."Registration on $p_host failed: ".
				       $nap->error.RESET."\n";
				next;
			}
		}
	}
	return 0;
}

#-------------------------
# display help und exit(0)
#-------------------------
sub help {

	my $help = <<HELP;

AutoNap 0.1.3 - an automatizable Napster-client
usage: $0 [options] 
options:	-d  <dir>       where to store downloads
		-f  <file>      read options from <file>
		-F  <length>    minimum filelength of song required
		-h      	print out this help
		-b  <bitrate>   set minimum bitrate to <bitrate>
		-l  <linespeed> set minimum linespeed to <linespeed>
		-m              specifies whether server (either passed by -S switch
			        or through configfile) is a metaserver or not
		-n  <network>   try to connect to preferably this <network>
		-p  <pass>      login with password <pass>
		-P  <player>    specify mpg3-player unless you use 'mpg123'
		-r              toggle random mode
		-R              register account, you should also give -n switch
		-x  <patterns>  random download but ignoring <patterns>            
		-s  <pattern>   search for pattern after startup
		-S  <server>    connect to <server>
		-u  <user>      login with username <user>
				only makes sense along with -p

		<file>		configfile that specifies AutoNap's behaviour
		<bitrate>       minimum bitrate of files to be returned on a search
		<linespeed>     user's minimum linespeed 
		<pattern>	search for song files containing this pattern,
				multiword pattern embraces in quotes
		<network>       one of the opennap networks, such as MusicCity or DJNap
		<server>        server in the form <address>:<port>
		<user>		your Napster-username
		<pass>		password according to your username

HELP
	print $help;
	exit(0);
}


__END__

=head1 NAME

AutoNap.pl - an interacting and non-interacting Napster/OpenNap client

=head1 SYNOPSIS

AutoNap.pl [options] 

=head2 options

-h

-R -N

[-f] [-p -u -m -S -n -P -r -x -l -b -d]

=head1 DESCRIPTION

AutoNap.pl is a program written in Perl which will connect to an OpenNap compatible network (Napster is one of those), search for given songs and download them.

It is intended either for interacting mode using a console color-interface or running unattendedly by following the directives given in a configfile. As for now, you will have to confine yourself to the interacting mode. Yet, I am doing my best to with implementing the non-interacting mode as well as making the interactive interface as smooth and comfortable as possible. 

If you ever find a feature announced as being available and in fact it is not yet there, please take into account that this client isn't even one month old. Same applies to bugs of which AutoNap is probably particularly rich. Don't just wipe it off from your system: report the bugs instead to me and I'll fix it as soon as possible, often within less than 24h. Or: be an even better boy by fixing it yourself and send me the diff. ;-) 

=head1 ARGUMENTS

=over 4

=item -h 

Displays the help screen and exits.

=item -R

Register a new a accont. It is advisable to use -R only along with -N since AutoNap's serverlist is unsorted and thus you cannot predict which network it'll try to register you on.

=item -f <file>
  
Reads options from file. See L<autonap.conf(5)> for details on how to setting up one.

=item -u <user>

Tries to login to the network with <user>.

=item -p <pass>

Identifies with <pass> on login to a network.

=item -S <server>:<port>

Tries a login on <server> and <port>. <port> is mandatory when specifying a server.

=item -n <network>

Logins to the specifies <network>. This only works when AutoNap was able to retrieve a serverlist from which it can deduce the network a server is belonging to. This means that -n doesn't make sense along with -S.

=item -m 

Indicates that the given server is a meta-server which requires a slightly different approach on login.

=item -d <dir>

Store downloads into <dir>. The user running AutoNap naturally needs to have write permissions to this directory.

=item -b <bitrate>

Sets the default minimum bitrate to <bitrate>. That means that AutoNap will as default only find songs that have at least this bitrate. 

=item -l <linespeed>

Works just like the --bitrate option only that it affects the linespeed of songs to be found. This options can be overriden when being inside AutoNap by suffixing the searchpattern with ||<linespeed>.
Valide values for <linespeed> are: 
T3, T1, DSL, Cable, 128K, 64K, 56K, 33K, 28K, 14K and UNKNOWN.

=item -P <player>

Specifies which mp3-player should be used for streaming. Only necessary to be specified when different from mpg123.

=item -r 

Toggles random mode, that is: tries to act as 'randomly' on search es possible. Intended for non-interact mode which isn't yet done.

=item -x <patterns>

Do random searches but none containing one of the <patterns>. <patterns> is a double-colon separated list.

=item -s <pattern>

Search for songs containing <pattern>. This switch is incompatible with -r and thus ignored when random is set.

=head1 EXAMPLES

This will try to register you on a MusicCity server. It will prompt you for desired username and password:

=head2 AutoNap.pl -R -N MusicCity

Read options from /home/ethan/autonap.conf but only try MusicCity servers:

=head2 AutoNap.pl -f /home/ethan/autonap.conf -N MusicCity

Try to login to any server with user/pass foo/bar and set mp3player to mp3blaster:

=head2 AutoNap.pl -u foo -p bar -P mp3blaster 

And so on, and so on. When you are smart, you should know yourself which options make sense with each other and which don't. ;-)

=head1 INTERACTIVE MODE

When using interactive mode (which is currently the only available one), you see yourself presented with the server's MOTD after login if there is any. You then can control AutoNap through the built-in commands which are listed to you when typing 'help'. Typing 'help <command>' gives you further advice on a particular command.

Some of the commands take arguments, some are without and some may take arguments. Commands without any further arguments are 'downs' and 'quit'. 'last' and 'help' can take arguments and for all other commands an argument must be supplied.

Prefixing a command with a '!' will invoke the following string to be interpreted as a system command. However, shell built-ins do not work, in order to execute these (things such as 'cd' etc.) you can do (for the bash) 
	
=head2 !bash -c 'cd /tmp'

As this is clumsy and hardly a win over switching to another shell, you can just start a new shell by typing

=head2 !bash

and then comfortably doing all your stuff there. B<exit> will return you to AutoNap.

=head1 PAGING

When browsing other user's song archive or retrieving results from a search, AutoNap wont scroll through to the bottom without you even catching a glimpse of the song. It will pause after (currently) 25 songs and ask you to press a key to go on scrolling or hitting 'q' to quit the list prematurely. There are two more keys you can press, 'g' and 'x'. 

'g' is useful when you just spied a song on the current page you badly need to have and you can't wait to scroll the list to the bottom or taking the pain on you to hit 'q' and then entering 'get <num>'. So it'll prompt you for this number, trying to initiate the download and return into paging mode. 'x' (short for eB<x>ecute and eB<x>it;-) does the same but wont return and instead fall back to the AutoNap shell.  

Both 'g' and 'x' wont affect 'last' in any way and prematurely leaving the list with 'q' will still 
cache the result, so you can rebrose that at a later time.
In case you mistakenly hit 'x' or 'g' and you do not want to download any of the songs, just hit RETURN without any input and it'll savely make you leave the prompt without the need of downloading anything.

=head1 AUTHOR AND COPYRIGHT

Tassilo von Parseval
<Tassilo.Parseval@post.RWTH-Aachen.de>.

Copyright (c)  2001 Tassilo von Parseval. All rights reserved.  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License (GPL).
				   
=cut



